@@ -1,12 +1,12 @@
+import copy
 from collections import OrderedDict
 import re
 import shutil
 import subprocess
 import random
+from contextlib import suppress
 
-from astropy import units as u
 from panoptes.pocs.camera.camera import AbstractCamera  # noqa
-from panoptes.pocs.camera.camera import AbstractGPhotoCamera  # noqa
 
 from panoptes.pocs.utils.logger import get_logger
 from panoptes.utils import error
@@ -45,13 +45,27 @@ def list_connected_cameras():
     return ports
 
 
-def create_cameras_from_config(*args, **kwargs):
+def create_cameras_from_config(config=None,
+                               cameras=None,
+                               auto_primary=True,
+                               recreate_existing=False,
+                               *args, **kwargs):
     """Create camera object(s) based on the config.
 
     Creates a camera for each camera item listed in the config. Ensures the
     appropriate camera module is loaded.
 
     Args:
+        config (dict or None): A config object for a camera or None to lookup in
+            config-server.
+        cameras (list of panoptes.pocs.camera.Camera or None): A list of camera
+            objects or None.
+        auto_primary (bool): If True, when no camera is marked as the primary camera,
+            the first camera in the list will be used as primary. Default True.
+        recreate_existing (bool): If True, a camera object will be recreated if an
+            existing camera with the same `uid` is already assigned. Should currently
+            only affect cameras that use the `sdk` (i.g. not DSLRs). Default False
+            raises an exception if camera is already assigned.
         *args (list): Passed to `get_config`.
         **kwargs (dict): Can pass a `cameras` object that overrides the info in
             the configuration file. Can also pass `auto_detect`(bool) to try and
@@ -67,26 +81,21 @@ def create_cameras_from_config(*args, **kwargs):
             auto_detect=True and no cameras are found.
         error.PanError: Description
     """
+    camera_config = config or get_config('cameras', *args, **kwargs)
 
-    config = get_config(*args, **kwargs)
-
-    # Helper method to first check kwargs then config
-    def kwargs_or_config(item, default=None):
-        return kwargs.get(item, config.get(item, default))
-
-    cameras = OrderedDict()
-    camera_info = kwargs_or_config('cameras')
-    if not camera_info:
+    if not camera_config:
         # cameras section either missing or empty
         logger.info('No camera information in config.')
-        return cameras
+        return None
 
-    logger.debug(f"Camera config: {camera_info}")
-
-    auto_detect = camera_info.get('auto_detect', False)
+    logger.debug(f"camera_config={camera_config!r}")
+    camera_defaults = camera_config.get('defaults', dict())
 
+    cameras = cameras or OrderedDict()
     ports = list()
 
+    auto_detect = camera_defaults.get('auto_detect', False)
+
     # Lookup the connected ports
     if auto_detect:
         logger.debug("Auto-detecting ports for cameras")
@@ -96,23 +105,18 @@ def create_cameras_from_config(*args, **kwargs):
             logger.warning(e)
 
         if len(ports) == 0:
-            raise error.CameraNotFound(
-                msg="No cameras detected. For testing, use camera simulator.")
+            raise error.CameraNotFound(msg="No cameras detected. For testing, use camera simulator.")
         else:
-            logger.debug(f"Detected Ports: {ports}")
+            logger.debug(f"Detected ports={ports!r}")
 
     primary_camera = None
 
-    # Different models require different connections methods.
-    model_requires = {
-        'canon_gphoto2': 'port',
-        'sbig': 'serial_number',
-        'zwo': 'serial_number',
-        'fli': 'serial_number',
-    }
+    device_info = camera_config['devices']
+    for cam_num, cfg in enumerate(device_info):
+        # Get a copy of the camera defaults and update with device config.
+        device_config = camera_defaults.copy()
+        device_config.update(cfg)
 
-    device_info = camera_info['devices']
-    for cam_num, device_config in enumerate(device_info):
         cam_name = device_config.setdefault('name', f'Cam{cam_num:02d}')
 
         # Check for proper connection method.
@@ -127,116 +131,49 @@ def create_cameras_from_config(*args, **kwargs):
                 continue
         elif model == 'simulator':
             device_config['port'] = f'usb:999,{random.randint(0, 1000):03d}'
-        else:
-            try:
-                # This is either `port` or `serial_number`.
-                connect_method = model_requires[model]
-                connect_value = device_config[connect_method]
-                device_config[connect_method] = connect_value
-            except KeyError as e:
-                logger.warning(f"Camera error: connect_method missing for {model}: {e!r}")
 
         logger.debug(f'Creating camera: {model}')
 
         try:
-            module = load_module(f'panoptes.pocs.camera.{model}')
-            logger.debug(f'Camera module: {module}')
-            # Create the camera object
-            cam = module.Camera(**device_config)
+            module = load_module(model)
+            logger.debug(f'Camera module: module={module!r}')
+
+            if recreate_existing:
+                with suppress(AttributeError):
+                    module._assigned_cameras = set()
+
+            # We either got a class or a module.
+            if callable(module):
+                camera = module(**device_config)
+            else:
+                if hasattr(module, 'Camera'):
+                    camera = module.Camera(**device_config)
+                else:
+                    raise error.NotFound(f'module={module!r} does not have a Camera object')
         except error.NotFound:
             logger.error(f"Cannot find camera module with config: {device_config}")
         except Exception as e:
-            logger.error(f"Cannot create camera type: {device_config['model']} {e}")
+            logger.error(f"Cannot create camera type: {model} {e}")
         else:
-            is_primary = ''
-            if camera_info.get('primary', '') == cam.uid:
-                cam.is_primary = True
-                primary_camera = cam
-                is_primary = ' [Primary]'
+            # Check if the config specified a primary camera and if it matches.
+            if camera.uid == camera_config.get('primary'):
+                camera.is_primary = True
+                primary_camera = camera
 
-            logger.debug(f"Camera created: {cam.name} {cam.uid}{is_primary}")
+            logger.debug(f"Camera created: camera={camera!r}")
 
-            cameras[cam_name] = cam
+            cameras[cam_name] = camera
 
     if len(cameras) == 0:
         raise error.CameraNotFound(msg="No cameras available")
 
     # If no camera was specified as primary use the first
-    if primary_camera is None:
+    if primary_camera is None and auto_primary:
+        logger.info(f'No primary camera given, assigning the first camera (auto_primary={auto_primary!r})')
         primary_camera = list(cameras.values())[0]  # First camera
         primary_camera.is_primary = True
 
-    logger.debug(f"Primary camera: {primary_camera}")
-    logger.debug(f"{len(cameras)} cameras created")
-
-    return cameras
-
-
-def create_camera_simulator(num_cameras=2):
-    """Create simulator camera object(s).
-
-    Args:
-        num_cameras (int): The number of simulated cameras to create, default 2.
-
-    Returns:
-        OrderedDict: An ordered dictionary of created camera objects, with the
-            camera name as key and camera instance as value. Returns an empty
-            OrderedDict if there is no camera configuration items.
-
-    Raises:
-        error.CameraNotFound: Raised if camera cannot be found at specified port or if
-            auto_detect=True and no cameras are found.
-    """
-    if num_cameras == 0:
-        raise error.CameraNotFound(msg="No cameras available")
-
-    cameras = OrderedDict()
-
-    # Set up a simulated camera with fully configured simulated focuser.
-    device_config = {
-        'model': 'simulator',
-        'port': '/dev/camera/simulator',
-        'focuser': {'model': 'simulator',
-                    'focus_port': '/dev/ttyFAKE',
-                    'initial_position': 20000,
-                    'autofocus_range': (40, 80),
-                    'autofocus_step': (10, 20),
-                    'autofocus_seconds': 0.1,
-                    'autofocus_size': 500},
-        'filterwheel': {'model': 'simulator',
-                        'filter_names': ['one', 'deux', 'drei', 'quattro'],
-                        'move_time': 0.1 * u.second,
-                        'timeout': 0.5 * u.second},
-        'readout_time': 0.5,
-    }
-    logger.debug(f"SimulatorCamera config: {device_config=}")
-
-    primary_camera = None
-    for cam_num in range(num_cameras):
-        cam_name = f'SimCam{cam_num:02d}'
-
-        logger.debug(f'Using camera simulator {cam_name}')
-
-        camera_model = device_config['model']
-        logger.debug(f'Creating camera: {camera_model}')
-
-        module = load_module(f'panoptes.pocs.camera.{camera_model}')
-        logger.debug(f'Camera module: {module}')
-
-        # Create the camera object
-        cam = module.Camera(name=cam_name, **device_config)
-
-        is_primary = ''
-        if cam_num == 0:
-            cam.is_primary = True
-            primary_camera = cam
-            is_primary = ' [Primary]'
-
-        logger.debug(f"Camera created: {cam.name} {cam.uid}{is_primary}")
-
-        cameras[cam_name] = cam
-
-    logger.debug(f"Primary camera: {primary_camera}")
-    logger.debug(f"{len(cameras)} cameras created")
+    logger.info(f"Primary camera: {primary_camera}")
+    logger.success(f"{len(cameras)} cameras created")
 
     return cameras
