@@ -1,14 +1,13 @@
 import os
-import pytest
 
+import pytest
 from astropy import units as u
 from astropy.coordinates import EarthLocation
-
 from panoptes.pocs.mount.bisque import Mount
+from panoptes.pocs.utils.theskyx import TheSkyX
 from panoptes.utils.config.client import get_config
-from panoptes.utils import altaz_to_radec
-from panoptes.utils import current_time
-from panoptes.utils.theskyx import TheSkyX
+from panoptes.utils.time import current_time
+from panoptes.utils.utils import altaz_to_radec
 
 pytestmark = pytest.mark.skipif(TheSkyX().is_connected is False, reason="TheSkyX is not connected")
 
