@@ -8,9 +8,10 @@ from contextlib import suppress
 
 import astropy.units as u
 from astropy.io import fits
+import requests
 
-from panoptes.pocs.camera.simulator import Camera as SimCamera
-from panoptes.pocs.camera.simulator_sdk import Camera as SimSDKCamera
+from panoptes.pocs.camera.simulator.dslr import Camera as SimCamera
+from panoptes.pocs.camera.simulator.ccd import Camera as SimSDKCamera
 from panoptes.pocs.camera.sbig import Camera as SBIGCamera
 from panoptes.pocs.camera.sbigudrv import SBIGDriver, INVALID_HANDLE_VALUE
 from panoptes.pocs.camera.fli import Camera as FLICamera
@@ -27,45 +28,34 @@ from panoptes.utils.config.client import get_config
 from panoptes.utils.config.client import set_config
 
 from panoptes.pocs.camera import create_cameras_from_config
-from panoptes.pocs.camera import create_camera_simulator
-
-focuser_params = {
-    'model': 'simulator',
-    'focus_port': '/dev/ttyFAKE',
-    'initial_position': 20000,
-    'autofocus_range': (40, 80),
-    'autofocus_step': (10, 20),
-    'autofocus_seconds': 0.1,
-    'autofocus_size': 500,
-    'autofocus_keep_files': False
-}
-
-filterwheel_params = {
-    'model': 'simulator',
-    'filter_names': ['one', 'deux', 'drei', 'quattro'],
-    'move_time': 0.1,
-    'timeout': 0.5
-}
-
-serial_number_params = 'SSC101'
+from panoptes.utils.serializers import to_json
 
 
-@pytest.fixture(scope='function', params=[
+@pytest.fixture(scope='module', params=[
     pytest.param([SimCamera, dict()]),
-    pytest.param([SimCamera, dict(focuser=focuser_params)]),
-    pytest.param([SimCamera, dict(filterwheel=filterwheel_params)]),
-    pytest.param([SimSDKCamera, dict(serial_number=serial_number_params)]),
+    pytest.param([SimCamera, get_config('cameras.devices[0]')]),
+    pytest.param([SimCamera, get_config('cameras.devices[1]')]),
+    pytest.param([SimCamera, get_config('cameras.devices[2]')]),
+    pytest.param([SimSDKCamera, get_config('cameras.devices[3]')]),
     pytest.param([SBIGCamera, 'sbig'], marks=[pytest.mark.with_camera]),
     pytest.param([FLICamera, 'fli'], marks=[pytest.mark.with_camera]),
     pytest.param([ZWOCamera, 'zwo'], marks=[pytest.mark.with_camera]),
 ], ids=[
-    'simulator', 'simulator_focuser', 'simulator_filterwheel', 'simulator_sdk',
-    'sbig', 'fli', 'zwo'
+    'dslr',
+    'dslr.00',
+    'dslr.focuser.cooling.00',
+    'dslr.filterwheel.cooling.00',
+    'ccd.filterwheel.cooling.00',
+    'sbig',
+    'fli',
+    'zwo'
 ])
 def camera(request):
     CamClass = request.param[0]
     cam_params = request.param[1]
 
+    camera = None
+
     if isinstance(cam_params, dict):
         # Simulator
         camera = CamClass(**cam_params)
@@ -76,43 +66,50 @@ def camera(request):
                 camera = CamClass(**cam_config)
                 break
 
-    camera.logger.debug(f'Yielding camera {camera}')
+    # Wait for cooled camera
+    if camera.is_cooled_camera:
+        assert not camera.is_temperature_stable
+        # Wait for cooling
+        while not camera.is_temperature_stable:
+            time.sleep(2)
+        assert camera.is_temperature_stable
+
     assert camera.is_ready
+    camera.logger.debug(f'Yielding camera {camera}')
     yield camera
 
     # simulator_sdk needs this explicitly removed for some reason.
+    # SDK Camera class destructor *should* be doing this when the fixture goes out of scope.
     with suppress(AttributeError):
         type(camera)._assigned_cameras.discard(camera.uid)
 
 
-@pytest.fixture(scope='function')
+@pytest.fixture(scope='module')
 def counter(camera):
     return {'value': 0}
 
 
-@pytest.fixture(scope='function')
+@pytest.fixture(scope='module')
 def patterns(camera, images_dir):
-    patterns = {'final': os.path.join(images_dir, 'focus', camera.uid, '*',
-                                      ('*_final.' + camera.file_extension)),
-                'fine_plot': os.path.join(images_dir, 'focus', camera.uid, '*',
-                                          'fine_focus.png'),
-                'coarse_plot': os.path.join(images_dir, 'focus', camera.uid, '*',
-                                            'coarse_focus.png')}
+    base_dir = os.path.join(images_dir, 'focus', camera.uid, '*')
+    patterns = {
+        'final': os.path.join(base_dir, ('*-final.' + camera.file_extension)),
+        'fine_plot': os.path.join(base_dir, 'fine-focus.png'),
+        'coarse_plot': os.path.join(base_dir, 'coarse-focus.png')
+    }
     return patterns
 
 
-def test_create_camera_simulator():
-    cameras = create_camera_simulator()
-    assert len(cameras) == 2
-
-    cameras = create_camera_simulator()
-    assert len(cameras) == 2
-
-    with pytest.raises(error.CameraNotFound):
-        create_camera_simulator(num_cameras=0)
+def reset_conf(config_host, config_port):
+    url = f'http://{config_host}:{config_port}/reset-config'
+    response = requests.post(url,
+                             data=to_json({'reset': True}),
+                             headers={'Content-Type': 'application/json'}
+                             )
+    assert response.ok
 
 
-def test_create_cameras_from_config_no_autodetect():
+def test_create_cameras_from_config_no_autodetect(config_host, config_port):
     set_config('cameras.auto_detect', False)
     set_config('cameras.devices', [
         dict(model='canon_gphoto2', port='/dev/fake01'),
@@ -122,17 +119,21 @@ def test_create_cameras_from_config_no_autodetect():
     with pytest.raises(error.CameraNotFound):
         create_cameras_from_config()
 
+    reset_conf(config_host, config_port)
 
-def test_create_cameras_from_config_autodetect():
-    set_config('cameras.auto_detect', True)
+
+def test_create_cameras_from_config_autodetect(config_host, config_port):
+    set_config('cameras.defaults.auto_detect', True)
     with pytest.raises(error.CameraNotFound):
         create_cameras_from_config()
+    reset_conf(config_host, config_port)
 
 
 # Hardware independent tests, mostly use simulator:
 
 def test_sim_create_focuser():
-    sim_camera = SimCamera(focuser={'model': 'simulator', 'focus_port': '/dev/ttyFAKE'})
+    sim_camera = SimCamera(focuser={'model': 'panoptes.pocs.focuser.simulator.Focuser',
+                                    'focus_port': '/dev/ttyFAKE'})
     assert isinstance(sim_camera.focuser, Focuser)
 
 
@@ -143,21 +144,20 @@ def test_sim_passed_focuser():
 
 
 def test_sim_bad_focuser():
-    with pytest.raises((NotFound)):
+    with pytest.raises(NotFound):
         SimCamera(focuser={'model': 'NOTAFOCUSER'})
 
 
 def test_sim_worse_focuser():
-    sim_camera = SimCamera(focuser='NOTAFOCUSER')
-    # Will log an error but raise no exceptions
-    assert sim_camera.focuser is None
+    with pytest.raises(NotFound):
+        sim_camera = SimCamera(focuser='NOTAFOCUSER')
 
 
 def test_sim_string():
     sim_camera = SimCamera()
-    assert str(sim_camera) == 'Simulated Camera ({}) on None'.format(sim_camera.uid)
+    assert str(sim_camera) == f'Simulated Camera ({sim_camera.uid})'
     sim_camera = SimCamera(name='Sim', port='/dev/ttyFAKE')
-    assert str(sim_camera) == 'Sim ({}) on /dev/ttyFAKE'.format(sim_camera.uid)
+    assert str(sim_camera) == f'Sim ({sim_camera.uid}) port=/dev/ttyFAKE'
 
 
 def test_sim_file_extension():
@@ -180,15 +180,16 @@ def test_sdk_no_serial_number():
 
 
 def test_sdk_camera_not_found():
-    with pytest.raises(error.PanError):
+    with pytest.raises(error.InvalidConfig):
         SimSDKCamera(serial_number='SSC404')
 
 
 def test_sdk_already_in_use():
-    sim_camera = SimSDKCamera(serial_number='SSC999')
+    serial_number = get_config('cameras.devices[-1].serial_number')
+    sim_camera = SimSDKCamera(serial_number=serial_number)
     assert sim_camera
     with pytest.raises(error.PanError):
-        SimSDKCamera(serial_number='SSC999')
+        SimSDKCamera(serial_number=serial_number)
 
 
 # Hardware independent tests for SBIG camera
@@ -253,8 +254,8 @@ def test_is_cooled(camera):
 
 def test_set_target_temperature(camera):
     if camera.is_cooled_camera:
-        camera._target_temperature = 10 * u.Celsius
-        assert abs(camera._target_temperature - 10 * u.Celsius) < 0.5 * u.Celsius
+        camera.target_temperature = 10 * u.Celsius
+        assert abs(camera.target_temperature - 10 * u.Celsius) < 0.5 * u.Celsius
     else:
         pytest.skip("Camera {} doesn't implement temperature control".format(camera.name))
 
@@ -301,7 +302,9 @@ def test_is_temperature_stable(camera):
     if camera.is_cooled_camera:
         camera.target_temperature = camera.temperature
         camera.cooling_enabled = True
-        time.sleep(1)
+        while not camera.is_temperature_stable:
+            time.sleep(2)
+
         assert camera.is_temperature_stable
         camera.cooling_enabled = False
         assert not camera.is_temperature_stable
@@ -317,10 +320,10 @@ def test_exposure(camera, tmpdir):
     fits_path = str(tmpdir.join('test_exposure.fits'))
 
     assert not camera.is_exposing
+    assert camera.is_ready
     # A one second normal exposure.
-    exp_event = camera.take_exposure(filename=fits_path)
+    camera.take_exposure(filename=fits_path)
     assert camera.is_exposing
-    assert not exp_event.is_set()
     assert not camera.is_ready
     # By default take_exposure is non-blocking, need to give it some time to complete.
     if isinstance(camera, FLICamera):
@@ -329,7 +332,6 @@ def test_exposure(camera, tmpdir):
         time.sleep(5)
     # Output file should exist, Event should be set and camera should say it's not exposing.
     assert os.path.exists(fits_path)
-    assert exp_event.is_set()
     assert not camera.is_exposing
     assert camera.is_ready
     # If can retrieve some header data there's a good chance it's a valid FITS file
@@ -353,9 +355,32 @@ def test_exposure_blocking(camera, tmpdir):
     assert header['IMAGETYP'] == 'Light Frame'
 
 
+def test_long_exposure_blocking(camera, tmpdir):
+    """
+    Tests basic take_exposure functionality
+    """
+    fits_path = str(tmpdir.join('test_long_exposure_blocking.fits'))
+    original_timeout = camera._timeout
+    original_readout = camera._readout_time
+    try:
+        camera._timeout = 1
+        camera._readout_time = 0.5
+        assert not camera.is_exposing
+        assert camera.is_ready
+        seconds = 2 * (camera._timeout + camera._readout_time)
+        camera.take_exposure(filename=fits_path, seconds=seconds, blocking=True)
+        # Output file should exist, Event should be set and camera should say it's not exposing.
+        assert os.path.exists(fits_path)
+        assert not camera.is_exposing
+        assert camera.is_ready
+    finally:
+        camera._timeout = original_timeout
+        camera._readout_time = original_readout
+
+
 def test_exposure_dark(camera, tmpdir):
     """
-    Tests taking a dark. At least for now only SBIG cameras do this.
+    Tests taking a dark.
     """
     fits_path = str(tmpdir.join('test_exposure_dark.fits'))
     # A 1 second dark exposure
@@ -365,6 +390,14 @@ def test_exposure_dark(camera, tmpdir):
     header = fits_utils.getheader(fits_path)
     assert header['EXPTIME'] == 1.0
     assert header['IMAGETYP'] == 'Dark Frame'
+    with suppress(AttributeError):
+        if not camera.can_take_internal_darks and camera.filterwheel._dark_position:
+            # Filterwheel should have moved to 'blank' position due to dark exposure.
+            assert camera.filterwheel.current_filter == 'blank'
+            fits_path2 = str(tmpdir.join('test_exposure_dark_light.fits'))
+            camera.take_exposure(filename=fits_path2, blocking=True)
+            # Filterwheel should have moved back to most recent non opaque filter now.
+            assert camera.filterwheel.current_filter == 'one'
 
 
 def test_exposure_collision(camera, tmpdir):
@@ -380,12 +413,17 @@ def test_exposure_collision(camera, tmpdir):
     fits_path_1 = str(tmpdir.join('test_exposure_collision1.fits'))
     fits_path_2 = str(tmpdir.join('test_exposure_collision2.fits'))
     camera.take_exposure(2 * u.second, filename=fits_path_1)
+    camera.logger.log('testing', 'Exposure 1 started')
     with pytest.raises(error.PanError):
         camera.take_exposure(1 * u.second, filename=fits_path_2)
-    if isinstance(camera, FLICamera):
-        time.sleep(10)
-    else:
-        time.sleep(5)
+    camera.logger.log('testing', 'Exposure 2 collided')
+    # Wait for exposure.
+    while camera.is_exposing:
+        time.sleep(0.5)
+    # Wait for readout on file.
+    while not os.path.exists(fits_path_1):
+        time.sleep(0.5)
+
     assert os.path.exists(fits_path_1)
     assert not os.path.exists(fits_path_2)
     assert fits_utils.getval(fits_path_1, 'EXPTIME') == 2.0
@@ -423,17 +461,16 @@ def test_exposure_not_connected(camera):
 
 
 def test_exposure_moving(camera, tmpdir):
-    if not camera.filterwheel:
+    if camera.filterwheel is None:
         pytest.skip("Camera does not have a filterwheel")
     fits_path_1 = str(tmpdir.join('test_not_moving.fits'))
     fits_path_2 = str(tmpdir.join('test_moving.fits'))
     camera.filterwheel.position = 1
-    exp_event = camera.take_exposure(filename=fits_path_1)
-    exp_event.wait()
+    exp_event = camera.take_exposure(filename=fits_path_1, blocking=True)
     assert os.path.exists(fits_path_1)
     move_event = camera.filterwheel.move_to(2)
     with pytest.raises(error.PanError):
-        camera.take_exposure(filename=fits_path_2)
+        camera.take_exposure(filename=fits_path_2, blocking=True)
     move_event.wait()
     assert not os.path.exists(fits_path_2)
 
@@ -444,11 +481,9 @@ def test_exposure_timeout(camera, tmpdir, caplog):
     """
     fits_path = str(tmpdir.join('test_exposure_timeout.fits'))
     # Make timeout extremely short to force a timeout error
-    original_timeout = camera._timeout
-    camera._timeout = 0.01
     # This should result in a timeout error in the poll thread, but the exception won't
     # be seen in the main thread. Can check for logged error though.
-    exposure_event = camera.take_exposure(seconds=2.0, filename=fits_path)
+    readout_thread = camera.take_exposure(seconds=2.0, filename=fits_path, timeout=0.01)
 
     # Wait for it all to be over.
     time.sleep(4)
@@ -458,8 +493,8 @@ def test_exposure_timeout(camera, tmpdir, caplog):
     # Should be no data file, camera should not be exposing, and exposure event should be set
     assert not os.path.exists(fits_path)
     assert not camera.is_exposing
-    assert exposure_event is camera._exposure_event
-    assert exposure_event.is_set()
+    assert not readout_thread.is_alive()
+    assert not camera._is_exposing_event.is_set()
 
 
 def test_observation(camera, images_dir):
@@ -469,13 +504,29 @@ def test_observation(camera, images_dir):
     field = Field('Test Observation', '20h00m43.7135s +22d42m39.0645s')
     observation = Observation(field, exptime=1.5 * u.second)
     observation.seq_time = '19991231T235959'
-    camera.take_observation(observation, headers={})
-    time.sleep(7)
+    observation_event = camera.take_observation(observation)
+    while not observation_event.is_set():
+        camera.logger.trace(f'Waiting for observation event from inside test.')
+        time.sleep(1)
     observation_pattern = os.path.join(images_dir, 'TestObservation',
                                        camera.uid, observation.seq_time, '*.fits*')
     assert len(glob.glob(observation_pattern)) == 1
-    for fn in glob.glob(observation_pattern):
-        os.remove(fn)
+
+
+def test_observation_headers_and_blocking(camera, images_dir):
+    """
+    Tests functionality of take_observation()
+    """
+    field = Field('Test Observation', '20h00m43.7135s +22d42m39.0645s')
+    observation = Observation(field, exptime=1.5 * u.second)
+    observation.seq_time = '19991231T235559'
+    camera.take_observation(observation, headers={'field_name': 'TESTVALUE'}, blocking=True)
+    observation_pattern = os.path.join(images_dir, 'TestObservation',
+                                       camera.uid, observation.seq_time, '*.fits*')
+    image_files = glob.glob(observation_pattern)
+    assert len(image_files) == 1
+    headers = fits_utils.getheader(image_files[0])
+    assert fits_utils.getval(image_files[0], 'FIELD') == 'TESTVALUE'
 
 
 def test_observation_nofilter(camera, images_dir):
@@ -484,18 +535,15 @@ def test_observation_nofilter(camera, images_dir):
     """
     field = Field('Test Observation', '20h00m43.7135s +22d42m39.0645s')
     observation = Observation(field, exptime=1.5 * u.second, filter_name=None)
-    observation.seq_time = '19991231T235959'
-    camera.take_observation(observation, headers={})
-    time.sleep(7)
+    observation.seq_time = '19991231T235159'
+    camera.take_observation(observation, blocking=True)
     observation_pattern = os.path.join(images_dir, 'TestObservation',
                                        camera.uid, observation.seq_time, '*.fits*')
     assert len(glob.glob(observation_pattern)) == 1
-    for fn in glob.glob(observation_pattern):
-        os.remove(fn)
 
 
 def test_autofocus_coarse(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus(coarse=True)
     autofocus_event.wait()
@@ -504,7 +552,7 @@ def test_autofocus_coarse(camera, patterns, counter):
 
 
 def test_autofocus_fine(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus()
     autofocus_event.wait()
@@ -513,7 +561,7 @@ def test_autofocus_fine(camera, patterns, counter):
 
 
 def test_autofocus_fine_blocking(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus(blocking=True)
     assert autofocus_event.is_set()
@@ -522,7 +570,7 @@ def test_autofocus_fine_blocking(camera, patterns, counter):
 
 
 def test_autofocus_with_plots(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus(make_plots=True)
     autofocus_event.wait()
@@ -532,7 +580,7 @@ def test_autofocus_with_plots(camera, patterns, counter):
 
 
 def test_autofocus_coarse_with_plots(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus(coarse=True, make_plots=True)
     autofocus_event.wait()
@@ -542,7 +590,7 @@ def test_autofocus_coarse_with_plots(camera, patterns, counter):
 
 
 def test_autofocus_keep_files(camera, patterns, counter):
-    if not camera.focuser:
+    if camera.focuser is None:
         pytest.skip("Camera does not have a focuser")
     autofocus_event = camera.autofocus(keep_files=True)
     autofocus_event.wait()
@@ -550,17 +598,26 @@ def test_autofocus_keep_files(camera, patterns, counter):
     assert len(glob.glob(patterns['final'])) == counter['value']
 
 
+def test_autofocus_no_darks(camera, patterns, counter):
+    if camera.focuser is None:
+        pytest.skip("Camera does not have a focuser")
+    autofocus_event = camera.autofocus(keep_files=True, take_dark=False)
+    autofocus_event.wait()
+    counter['value'] += 1
+    assert len(glob.glob(patterns['final'])) == counter['value']
+
+
 def test_autofocus_no_size(camera):
     try:
         initial_focus = camera.focuser.position
     except AttributeError:
         pytest.skip("Camera does not have an exposed focuser attribute")
     initial_focus = camera.focuser.position
-    thumbnail_size = camera.focuser.autofocus_size
+    cutout_size = camera.focuser.autofocus_size
     camera.focuser.autofocus_size = None
     with pytest.raises(ValueError):
         camera.autofocus()
-    camera.focuser.autofocus_size = thumbnail_size
+    camera.focuser.autofocus_size = cutout_size
     assert camera.focuser.position == initial_focus
 
 
