@@ -5,7 +5,7 @@ from timeit import timeit
 from astropy import units as u
 
 from panoptes.pocs.filterwheel.simulator import FilterWheel as SimFilterWheel
-from panoptes.pocs.camera.simulator import Camera as SimCamera
+from panoptes.pocs.camera.simulator.dslr import Camera as SimCamera
 from panoptes.utils import error
 
 
@@ -17,7 +17,13 @@ def filterwheel():
     return sim_filterwheel
 
 
-# intialisation
+@pytest.fixture(scope='function')
+def filterwheel_with_blank():
+    sim_filterwheel = SimFilterWheel(filter_names=['blank', 'deux', 'drei', 'quattro'],
+                                     move_time=0.1 * u.second,
+                                     timeout=0.5 * u.second,
+                                     dark_position='blank')
+    return sim_filterwheel
 
 
 def test_init(filterwheel):
@@ -26,7 +32,7 @@ def test_init(filterwheel):
 
 
 def test_camera_init():
-    sim_camera = SimCamera(filterwheel={'model': 'simulator',
+    sim_camera = SimCamera(filterwheel={'model': 'panoptes.pocs.filterwheel.simulator.FilterWheel',
                                         'filter_names': ['one', 'deux', 'drei', 'quattro']})
     assert isinstance(sim_camera.filterwheel, SimFilterWheel)
     assert sim_camera.filterwheel.is_connected
@@ -56,7 +62,7 @@ def test_with_no_name():
 
 def test_model(filterwheel):
     model = filterwheel.model
-    assert model == 'simulator'
+    assert model == 'panoptes.pocs.filterwheel.simulator.FilterWheel'
     with pytest.raises(AttributeError):
         filterwheel.model = "Airfix"
 
@@ -164,15 +170,15 @@ def test_move_times(name, unidirectional, expected):
 
 
 def test_move_exposing(tmpdir):
-    sim_camera = SimCamera(filterwheel={'model': 'simulator',
+    sim_camera = SimCamera(filterwheel={'model': 'panoptes.pocs.filterwheel.simulator.FilterWheel',
                                         'filter_names': ['one', 'deux', 'drei', 'quattro']})
     fits_path = str(tmpdir.join('test_exposure.fits'))
-    exp_event = sim_camera.take_exposure(filename=fits_path, seconds=0.1)
+    readout_thread = sim_camera.take_exposure(filename=fits_path, seconds=0.1)
     with pytest.raises(error.PanError):
         # Attempt to move while camera is exposing
         sim_camera.filterwheel.move_to(2, blocking=True)
     assert sim_camera.filterwheel.position == 1  # Should not have moved
-    exp_event.wait()
+    readout_thread.join()
 
 
 def test_is_moving(filterwheel):
@@ -185,3 +191,26 @@ def test_is_moving(filterwheel):
     e.wait()
     assert not filterwheel.is_moving
     assert filterwheel.is_ready
+
+
+def test_move_dark(filterwheel, filterwheel_with_blank):
+    with pytest.raises(error.NotFound):
+        filterwheel.move_to_dark_position()
+    filterwheel_with_blank.move_to('deux', blocking=True)
+    assert filterwheel_with_blank.current_filter == 'deux'
+    filterwheel_with_blank.move_to_dark_position(blocking=True)
+    assert filterwheel_with_blank.current_filter == 'blank'
+
+
+def test_move_light(filterwheel, filterwheel_with_blank):
+    with pytest.raises(error.NotFound):
+        filterwheel.move_to_light_position()
+    with pytest.raises(error.NotFound):
+        # Won't have been set yet.
+        filterwheel_with_blank.move_to_light_position()
+    filterwheel_with_blank.move_to('deux', blocking=True)
+    assert filterwheel_with_blank.current_filter == 'deux'
+    filterwheel_with_blank.move_to_dark_position(blocking=True)
+    assert filterwheel_with_blank.current_filter == 'blank'
+    filterwheel_with_blank.move_to_light_position(blocking=True)
+    assert filterwheel_with_blank.current_filter == 'deux'
