@@ -14,12 +14,9 @@ from panoptes.utils.serializers import to_json
 
 import requests
 
-config_host = 'localhost'
-config_port = 6563
-url = f'http://{config_host}:{config_port}/reset-config'
 
-
-def reset_conf():
+def reset_conf(config_host, config_port):
+    url = f'http://{config_host}:{config_port}/reset-config'
     response = requests.post(url,
                              data=to_json({'reset': True}),
                              headers={'Content-Type': 'application/json'}
@@ -27,13 +24,13 @@ def reset_conf():
     assert response.ok
 
 
-def test_create_mount_simulator():
+def test_create_mount_simulator(config_host, config_port):
     # Use the simulator create function directly.
     mount = create_mount_simulator()
     assert isinstance(mount, AbstractMount) is True
 
 
-def test_create_mount_simulator_with_config():
+def test_create_mount_simulator_with_config(config_host, config_port):
     # Remove mount from list of simulators.
     set_config('simulator', hardware.get_all_names(without=['mount']))
     # But setting the driver to `simulator` should return simulator.
@@ -41,20 +38,20 @@ def test_create_mount_simulator_with_config():
 
     mount = create_mount_from_config()
     assert isinstance(mount, AbstractMount) is True
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_create_mount_without_mount_info():
+def test_create_mount_without_mount_info(config_host, config_port):
     # Set the mount config to none and then don't pass anything for error.
     set_config('mount', None)
     set_config('simulator', hardware.get_all_names(without=['mount']))
     with pytest.raises(error.MountNotFound):
         create_mount_from_config(mount_info=None)
 
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_create_mount_with_mount_info():
+def test_create_mount_with_mount_info(config_host, config_port):
     # Pass the mount info directly with nothing in config.
     mount_info = get_config('mount', default=dict())
     mount_info['driver'] = 'simulator'
@@ -64,10 +61,10 @@ def test_create_mount_with_mount_info():
     set_config('simulator', hardware.get_all_names(without=['mount']))
     assert isinstance(create_mount_from_config(mount_info=mount_info), AbstractMount) is True
 
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_create_mount_with_earth_location():
+def test_create_mount_with_earth_location(config_host, config_port):
     # Get location to pass manually.
     loc = create_location_from_config()
     # Set config to not have a location.
@@ -75,39 +72,39 @@ def test_create_mount_with_earth_location():
     set_config('simulator', hardware.get_all_names())
     assert isinstance(create_mount_from_config(earth_location=loc['earth_location']), AbstractMount) is True
 
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_create_mount_without_earth_location():
+def test_create_mount_without_earth_location(config_host, config_port):
     set_config('location', None)
     with pytest.raises(error.PanError):
         create_mount_from_config(earth_location=None)
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_bad_mount_port():
+def test_bad_mount_port(config_host, config_port):
     # Remove the mount from the list of simulators so it thinks we have a real one.
     simulators = get_config('simulator')
     with suppress(KeyError, AttributeError):
-        simulators.remove('mount')
+        simulators.pop('mount')
     set_config('simulator', simulators)
 
     # Set a bad port, which should cause a fail before actual mount creation.
     set_config('mount.serial.port', 'foobar')
     with pytest.raises(error.MountNotFound):
         create_mount_from_config()
-    reset_conf()
+    reset_conf(config_host, config_port)
 
 
-def test_bad_mount_driver():
+def test_bad_mount_driver(config_host, config_port):
     # Remove the mount from the list of simulators so it thinks we have a real one.
     simulators = get_config('simulator')
     with suppress(KeyError, AttributeError):
-        simulators.remove('mount')
+        simulators.pop('mount')
     set_config('simulator', simulators)
 
     # Set a bad port, which should cause a fail before actual mount creation.
     set_config('mount.serial.driver', 'foobar')
     with pytest.raises(error.MountNotFound):
         create_mount_from_config()
-    reset_conf()
+    reset_conf(config_host, config_port)
