@@ -19,8 +19,8 @@ class PanLogger:
         self.padding = 0
         # Level Time_UTC Time_Local dynamic_padding Message
         self.fmt = "<lvl>{level:.1s}</lvl> " \
-                   "<light-blue>{time:MM-DD HH:mm:ss.ss!UTC}</>" \
-                   " <blue>({time:HH:mm:ss.ss})</> " \
+                   "<light-blue>{time:MM-DD HH:mm:ss.SSS!UTC}</>" \
+                   " <blue>({time:HH:mm:ss zz})</> " \
                    "| <c>{name} {function}:{line}{extra[padding]}</c> | " \
                    "<lvl>{message}</lvl>\n"
         self.handlers = dict()
@@ -86,7 +86,7 @@ def get_logger(console_log_file='panoptes.log',
             loguru_logger.remove(0)
 
         stderr_format = "<lvl>{level:.1s}</lvl> " \
-                        "<light-blue>{time:MM-DD HH:mm:ss.ss!UTC}</> " \
+                        "<light-blue>{time:MM-DD HH:mm:ss.SSS!UTC}</> " \
                         "<lvl>{message}</lvl>"
 
         stderr_id = loguru_logger.add(
